# -*- coding: utf-8 -*-
"""
Created on Wed Sep 28 10:13:23 2022

@author: lucenthoven
"""


import numpy as np
import qutip as qt
import matplotlib.pyplot as plt
# import matplotlib.font_manager as fm
import scipy as sc
from matplotlib import ticker

plt.close('all')
plt.style.use("seaborn-v0_8-paper")

import matplotlib.font_manager as fm
font_path = 'C:\Windows\Fonts\ARIALNB.TTF'  # the location of the font file
# font_path = "liberation-narrow-fonts-ttf-1.07.6\LiberationSansNarrow-Bold.ttf"
suptitlefont = fm.FontProperties(size = 14, fname=font_path)  # get the font based on the font_path
titlefont = fm.FontProperties(size = 12, fname=font_path)  # get the font based on the font_path
axfont = fm.FontProperties(size = 20, fname=font_path)  # get the font based on the font_path

fm.fontManager.addfont(font_path)
prop = fm.FontProperties(fname=font_path)
plt.rcParams['font.family'] = 'sans-serif'
plt.rcParams['font.sans-serif'] = "Arial"
plt.rcParams['font.weight'] = 'bold'
plt.rcParams['font.stretch'] = 'condensed'
plt.rcParams['xtick.labelsize']=20
plt.rcParams['ytick.labelsize']=20

data_y_w = np.genfromtxt("COMSOL_data/y_field_w/y_field_w_16_10_2023.csv", skip_header=5, delimiter=",")
data_x_w = np.genfromtxt("COMSOL_data/x_field_w/x_field_w_16_10_2023.csv", skip_header=5, delimiter=",")
# data_x_w_line = np.genfromtxt("COMSOL_data/x_field_w/x_field_plot_data_line.csv", skip_header=8, delimiter=",")
data_z_w = np.genfromtxt("COMSOL_data/z_field_w/z_field_ind_width_17_10_2023_m2.csv", skip_header=5, delimiter=",")
data_z_N = np.genfromtxt("COMSOL_data/z_field_turns/z_field_turns_17_10_2023_m2.csv", skip_header=5, delimiter=",")
data_z_inner_radius = np.genfromtxt("COMSOL_data/z_field_inner_rad/z_field_ind_inner_rad_17_10_2023_m2.csv", skip_header=5, delimiter=",")

w_y = data_y_w[:,1]
k_y = data_y_w[:,3]

w_x = data_x_w[:,1]
k_x = data_x_w[:,3]

#Plot point 4 for Z-coil
w_z = data_z_w[:,2]
k_zw = data_z_w[:,4]

N_z = data_z_N[:,0]
k_zN = data_z_N[:,3]

rad_z = data_z_inner_radius[:,1]
k_zrad = data_z_inner_radius[:,3]

plt.figure(figsize=(4,4))
plt.plot(w_y, abs(k_y), linewidth=2)
plt.xlabel("W[$\mathregular{\mu}$m]", fontproperties=axfont)
plt.ylabel("$\mathregular{k_x[G/A]}$", fontproperties=axfont)
plt.grid()
plt.tight_layout()
plt.savefig("figures/w_sweep_x.png", dpi=300)
plt.savefig("figures/w_sweep_x.eps", format="eps")

plt.figure(figsize=(4,4))
plt.plot(rad_z, abs(k_zrad), linewidth=2)
plt.xlabel("Inner Radius[$\mathregular{\mu}$m]", fontproperties=axfont)
plt.ylabel("$\mathregular{k_z[G/A]}$", fontproperties=axfont)
plt.grid()
plt.tight_layout()
plt.savefig("figures/inner_rad_z.png", dpi=300)
plt.savefig("figures/inner_rad_z.eps", format="eps")

plt.figure(figsize=(4,4))
plt.plot(w_z, abs(k_zw), linewidth=2)
plt.xlabel("W[$\mathregular{\mu}$m]", fontproperties=axfont)
plt.ylabel("$\mathregular{k_z[G/A]}$", fontproperties=axfont)
plt.grid()
plt.tight_layout()
plt.savefig("figures/w_sweep_z.png", dpi=300)
plt.savefig("figures/w_sweep_z.eps", format="eps")


plt.figure(figsize=(4,4))
plt.plot(N_z, abs(k_zN), linewidth=2)
plt.xlabel("$\mathregular{N_{turns}}$", fontproperties=axfont)
plt.ylabel("$\mathregular{k_z[G/A]}$", fontproperties=axfont)
plt.grid()
plt.tight_layout()
plt.savefig("figures/N_sweep_z.png", dpi=300)
plt.savefig("figures/N_sweep_z.eps", format="eps")


