# -*- coding: utf-8 -*-
"""
Created on Wed Sep  7 11:27:23 2022

@author: lucenthoven
"""

import numpy as np
import qutip as qt
import matplotlib.pyplot as plt
import scipy as sc
# from mpl_toolkits.axes_grid.inset_locator import (inset_axes, InsetPosition,
#                                                   mark_inset)
from matplotlib import ticker

from matplotlib import lines
from matplotlib.patches import Rectangle
from matplotlib.transforms import Bbox
import matplotlib.font_manager as fm
plt.style.use("seaborn-v0_8-paper")
# plt.style.use("seaborn-paper")

font_path = 'C:\Windows\Fonts\ARIALNB.TTF'  # the location of the font file
# font_path = "liberation-narrow-fonts-ttf-1.07.6\LiberationSansNarrow-Bold.ttf"
suptitlefont = fm.FontProperties(size = 14, fname=font_path)  # get the font based on the font_path
titlefont = fm.FontProperties(size = 12, fname=font_path)  # get the font based on the font_path
axfont = fm.FontProperties(size = 12, fname=font_path)  # get the font based on the font_path

fm.fontManager.addfont(font_path)
prop = fm.FontProperties(fname=font_path)
plt.rcParams['font.family'] = 'sans-serif'
plt.rcParams['font.sans-serif'] = "Arial"
plt.rcParams['font.weight'] = 'bold'
plt.rcParams['font.stretch'] = 'condensed'
# plt.style.use("seaborn-v0_8-paper")
plt.rcParams['xtick.labelsize']=11
plt.rcParams['ytick.labelsize']=11

plt.close('all')

# Define constants

y_e= 2.8031e6 # Gyromagnetic ratio Hz/G
y_c = 1.0705e3
y_n = 0.3077e3  # Units

k_coil = 710 # Coupling factor I->B
k_x = 290



D = 2.88e9

Bz = 2e3

f_re = 5e6
f_rc = 5e3
theta = np.pi

# b_e = 5
# b_n = 9.3

b_comp = 2.4
Ncells = 100

Fs = 50e6
e_bit = 84e-15

#%% Plotting the various coil simulations

#%% Getting the power dissipated by the electronics for generating a DC magnetic field
i_coil = b_comp/k_coil

r_coil = 500e-3
r_on = 500e-3
r_ic = 50e-3

p_dc_magnetic = i_coil **2 * (r_coil + r_on + Ncells * r_ic)


#%% Calculate NCO bits 

infidelity = 1e-5

Top_e = theta/(2*np.pi*f_re)
Top_n = theta/(2*np.pi*f_rc)

inf_e = infidelity
inf_n = infidelity

print("Infidelity e:", inf_e)
print("Infidelity n:", inf_n)

inf_ph_e = inf_e
inf_ph_n = inf_n


Nbits_e = np.ceil(np.log2((np.pi*Fs*Top_e)/np.arccos(np.sqrt(1-inf_ph_e)))-1)
Nbits_n = np.ceil(np.log2((np.pi*Fs*Top_n)/np.arccos(np.sqrt(1-inf_ph_n)))-1)

print("NCO e:", Nbits_e)
print("NCO n:", Nbits_n)

ferr_e = Fs/(2**(Nbits_e+1))
ferr_n = Fs/(2**(Nbits_n+1))

print("Ferr e, pur:", ferr_e)
print("Ferr n, pur:", ferr_n)

inf_ph_e_nco = 1-(np.cos(np.pi*Fs*Top_e/(2**(Nbits_e+1))))**2
inf_ph_n_nco = 1-(np.cos(np.pi*Fs*Top_n/(2**(Nbits_n+1))))**2


print("Infidelity NCO e, pur:", inf_ph_e_nco)
print("Infidelity NCO n, pur:", inf_ph_n_nco)

#Power dissipated 

p_nco_e = e_bit * Fs * Nbits_e
p_nco_n = e_bit * Fs * Nbits_n

print("Power NCO e:", p_nco_e)
print("Power NCO n:", p_nco_n)
print("total power:", p_nco_e+9*p_nco_n)


#%% Calculate analog power 

# i_req_e = b_e/k_x
# i_req_n = b_n/k_x 

# print(i_req_e)
# print(i_req_n)

#%% Compute power per unit cell as function of field inhomogeneity
Ncells=100
f_space = 10e6
Vdd = 1.1
Vsup = 0.1
D_eop = 0.1
b_comp = np.logspace(-1, 3, 100)
f_comp = b_comp*y_e
f_space_arr = f_space * np.ones([b_comp.shape[0]])
f_zeros = np.zeros([b_comp.shape[0]])

i_comp = b_comp/k_coil/2 #reduce in half since we have +/- control

p_dc_magnetic = i_comp **2 * (r_coil + r_on + Ncells * r_ic) + 100e-6
p_dc_magnetic_4k = i_comp **2 * (r_coil/4 + r_on/2 + Ncells * r_ic/4) + 100e-6


scaling_coeff = 2.5

Fs = np.max(np.c_[f_space_arr, f_comp], axis=1) * scaling_coeff
Fs_min = f_space_arr * scaling_coeff

infidelity = 1e-5

f_re = 5e6
f_rc = 5e3

Top_e = theta/(2*np.pi*f_re)
Top_n = theta/(2*np.pi*f_rc)

inf_e = infidelity
inf_n = infidelity

inf_ph_e = inf_e
inf_ph_n = inf_n

Nbits_e = np.ceil(np.log2((np.pi*Fs*Top_e)/np.arccos(np.sqrt(1-inf_ph_e)))-1)
Nbits_n = np.ceil(np.log2((np.pi*Fs*Top_n)/np.arccos(np.sqrt(1-inf_ph_n)))-1)
# Nbits_e = np.ceil(np.log2((Fs)/(f_re*np.sqrt(1-inf_ph_e)))-1)
# Nbits_n = np.ceil(np.log2((Fs)/(f_rc*np.sqrt(1-inf_ph_n)))-1)
# Nbits_n = np.ceil(np.log2((np.pi*Fs*Top_n)/np.arccos(np.sqrt(1-inf_ph_n)))-1)


#Power dissipated in NCOs with freq. compensation

p_nco_e = e_bit * Fs * Nbits_e
p_nco_n = e_bit * Fs * Nbits_n
ptot = p_nco_e+9*p_nco_n


Nbits_e_min = np.ceil(np.log2((np.pi*Fs_min*Top_e)/np.arccos(np.sqrt(1-inf_ph_e)))-1)
Nbits_n_min = np.ceil(np.log2((np.pi*Fs_min*Top_n)/np.arccos(np.sqrt(1-inf_ph_n)))-1)
# Nbits_e_min = np.ceil(np.log2((Fs_min)/(f_re*np.sqrt(1-inf_ph_e)))-1)
# Nbits_n_min = np.ceil(np.log2((Fs_min)/(f_rc*np.sqrt(1-inf_ph_n)))-1)

#Power dissipated in NCOs with mag. field compensation

p_nco_e_min = e_bit * Fs_min * Nbits_e_min
p_nco_n_min = e_bit * Fs_min * Nbits_n_min
ptot_min = p_nco_e_min+9*p_nco_n_min

#Miminum dissipation due to electron and carbon driving
p_e_op = f_re/(y_e*k_x) * Vdd * D_eop * np.ones([b_comp.shape[0]])
p_c_op = f_rc/(y_c*k_x) * Vsup * np.ones([b_comp.shape[0]])
p_static_ac_sig = p_e_op + p_c_op

fig, ax = plt.subplots(2, 1, sharex=True, figsize=(6,4))
ax[0].loglog(b_comp, ptot+p_static_ac_sig, label="Uncompensated field")
# plt.loglog(b_comp, p_dc_magnetic, label="Power dissipation magnetic field")
ax[0].loglog(b_comp, ptot_min+p_dc_magnetic_4k+p_static_ac_sig, label="Compensated field")
# plt.loglog(b_comp, ptot_min, label="Power dissipation Digital QCS compensated field")
# plt.loglog(b_comp, ptot_min+p_dc_magnetic, label="Power dissipation compensated field")
# ax[0].set_title("10 by 10 unit cells", fontproperties=titlefont)
ax[0].set_ylabel("Power dissipation [W]", fontproperties=axfont)
ax[0].set_title("10 by 10 unit cells", fontproperties=titlefont)
ax[0].set_ylim([0.5e-3, 1])
ax[0].set_xlim([1e-1, 1e3])
ax[0].grid()
ax[0].legend(fontsize="medium")


# ax[2].loglog(b_comp, p_e_op, label="Electron output amplifier")
# ax[2].loglog(b_comp, p_c_op, label="Nuclear output amplifier")
# ax[2].loglog(b_comp, ptot, label="NCO dissipation (Uncompensated field)")
# ax[2].loglog(b_comp, p_dc_magnetic_4k, label="DC magnetic field generation (Compensated field)")
# # plt.loglog(b_comp, p_dc_magnetic, label="Power dissipation magnetic field")

# # plt.loglog(b_comp, ptot_min, label="Power dissipation Digital QCS compensated field")
# # plt.loglog(b_comp, ptot_min+p_dc_magnetic, label="Power dissipation compensated field")

# ax[2].set_ylabel("Power dissipation [W]")
# ax[2].set_title("Power per component (10x10 unit cells)")
# ax[2].set_ylim([0.5e-3, 1])
# ax[2].set_xlim([1e-1, 1e3])
# ax[2].grid()
# ax[2].legend()


f_space = 10e6
Ncells=10000
b_comp = np.logspace(-1, 3, 100)
f_comp = b_comp*y_e
f_zeros = np.zeros([b_comp.shape[0]])

i_comp = b_comp/k_coil/2 #reduce in half since we have +/- control

p_dc_magnetic = i_comp **2 * (r_coil + r_on + Ncells * r_ic) + 100e-6
p_dc_magnetic_4k = i_comp **2 * (r_coil/4 + r_on/2 + Ncells * r_ic/4) + 100e-6


scaling_coeff = 2.5

Fs = (f_space + f_comp) * scaling_coeff
Fs_min = (f_space + f_zeros ) * scaling_coeff

infidelity = 1e-5

f_re = 5e6
f_rc = 5e3

Top_e = theta/(2*np.pi*f_re)
Top_n = theta/(2*np.pi*f_rc)

inf_e = infidelity
inf_n = infidelity

inf_ph_e = inf_e
inf_ph_n = inf_n

Nbits_e = np.ceil(np.log2((np.pi*Fs*Top_e)/np.arccos(np.sqrt(1-inf_ph_e)))-1)
Nbits_n = np.ceil(np.log2((np.pi*Fs*Top_n)/np.arccos(np.sqrt(1-inf_ph_n)))-1)
# Nbits_e = np.ceil(np.log2((Fs)/(f_re*np.sqrt(1-inf_ph_e)))-1)
# Nbits_n = np.ceil(np.log2((Fs)/(f_rc*np.sqrt(1-inf_ph_n)))-1)


#Power dissipated in NCOs with freq. compensation

p_nco_e = e_bit * Fs * Nbits_e
p_nco_n = e_bit * Fs * Nbits_n
ptot = p_nco_e+9*p_nco_n


Nbits_e_min = np.ceil(np.log2((np.pi*Fs_min*Top_e)/np.arccos(np.sqrt(1-inf_ph_e)))-1)
Nbits_n_min = np.ceil(np.log2((np.pi*Fs_min*Top_n)/np.arccos(np.sqrt(1-inf_ph_n)))-1)
# Nbits_e_min = np.ceil(np.log2((Fs_min)/(f_re*np.sqrt(1-inf_ph_e)))-1)
# Nbits_n_min = np.ceil(np.log2((Fs_min)/(f_rc*np.sqrt(1-inf_ph_n)))-1)

#Power dissipated in NCOs with mag. field compensation

p_nco_e_min = e_bit * Fs_min * Nbits_e_min
p_nco_n_min = e_bit * Fs_min * Nbits_n_min
ptot_min = p_nco_e_min+9*p_nco_n_min


ax[1].loglog(b_comp, ptot+p_static_ac_sig, label="Uncompensated field")
# plt.loglog(b_comp, p_dc_magnetic, label="Power dissipation magnetic field")
ax[1].loglog(b_comp, ptot_min+p_dc_magnetic_4k+p_static_ac_sig, label="Compensated field")
# plt.loglog(b_comp, ptot_min, label="Power dissipation Digital QCS compensated field")
# plt.loglog(b_comp, ptot_min+p_dc_magnetic, label="Power dissipation compensated field")
ax[1].set_xlabel("$B_{comp}$ [G]", fontproperties=axfont)
ax[1].set_ylabel("Power dissipation [W]", fontproperties=axfont)
ax[1].set_title("100 by 100 unit cells", fontproperties=titlefont)
ax[1].set_ylim([0.5e-3, 1])
ax[1].set_xlim([1e-1, 1e3])
ax[1].grid()
ax[1].legend(fontsize="medium")

fig.tight_layout()
fig.savefig("figures/power_dissipation_tradeoff.eps", format="eps")

#%% Compute power per unit cell as function of field inhomogeneity, add the hybrid compensation (both magnetic field and frequency)
Ncells=100
f_space = 10e6
Vdd = 1.1
Vsup = 0.1
D_eop = 0.1
b_comp = np.logspace(-1, 3, 100)
f_comp = b_comp*y_e
f_comp2 = b_comp/2*y_e
f_space_arr = f_space * np.ones([b_comp.shape[0]])
f_zeros = np.zeros([b_comp.shape[0]])

i_comp = b_comp/k_coil/2 #reduce in half since we have +/- control
i_comp2 = b_comp/k_coil/4 #reduce in half since we have +/- control

p_dc_magnetic = i_comp **2 * (r_coil + r_on + Ncells * r_ic) + 100e-6
p_dc_magnetic_4k = i_comp **2 * (r_coil/4 + r_on/2 + Ncells * r_ic/4) + 100e-6
p_dc_magnetic_4k2 = i_comp2 **2 * (r_coil/4 + r_on/2 + Ncells * r_ic/4) + 100e-6


scaling_coeff = 2.5

Fs = np.max(np.c_[f_space_arr, f_comp], axis=1) * scaling_coeff
Fs2 = np.max(np.c_[f_space_arr, f_comp2], axis=1) * scaling_coeff

Fs_min = f_space_arr * scaling_coeff


infidelity = 1e-5

f_re = 5e6
f_rc = 5e3

Top_e = theta/(2*np.pi*f_re)
Top_n = theta/(2*np.pi*f_rc)

inf_e = infidelity
inf_n = infidelity

inf_ph_e = inf_e
inf_ph_n = inf_n

Nbits_e = np.ceil(np.log2((np.pi*Fs*Top_e)/np.arccos(np.sqrt(1-inf_ph_e)))-1)
Nbits_n = np.ceil(np.log2((np.pi*Fs*Top_n)/np.arccos(np.sqrt(1-inf_ph_n)))-1)
Nbits_e2 = np.ceil(np.log2((np.pi*Fs2*Top_e)/np.arccos(np.sqrt(1-inf_ph_e)))-1)
Nbits_n2 = np.ceil(np.log2((np.pi*Fs2*Top_n)/np.arccos(np.sqrt(1-inf_ph_n)))-1)
# Nbits_e = np.ceil(np.log2((Fs)/(f_re*np.sqrt(1-inf_ph_e)))-1)
# Nbits_n = np.ceil(np.log2((Fs)/(f_rc*np.sqrt(1-inf_ph_n)))-1)
# Nbits_n = np.ceil(np.log2((np.pi*Fs*Top_n)/np.arccos(np.sqrt(1-inf_ph_n)))-1)


#Power dissipated in NCOs with freq. compensation

p_nco_e = e_bit * Fs * Nbits_e
p_nco_n = e_bit * Fs * Nbits_n
p_nco_e2 = e_bit * Fs2 * Nbits_e
p_nco_n2 = e_bit * Fs2 * Nbits_n
ptot = p_nco_e+9*p_nco_n
ptot2 = p_nco_e2+9*p_nco_n2


Nbits_e_min = np.ceil(np.log2((np.pi*Fs_min*Top_e)/np.arccos(np.sqrt(1-inf_ph_e)))-1)
Nbits_n_min = np.ceil(np.log2((np.pi*Fs_min*Top_n)/np.arccos(np.sqrt(1-inf_ph_n)))-1)
# Nbits_e_min = np.ceil(np.log2((Fs_min)/(f_re*np.sqrt(1-inf_ph_e)))-1)
# Nbits_n_min = np.ceil(np.log2((Fs_min)/(f_rc*np.sqrt(1-inf_ph_n)))-1)

#Power dissipated in NCOs with mag. field compensation

p_nco_e_min = e_bit * Fs_min * Nbits_e_min
p_nco_n_min = e_bit * Fs_min * Nbits_n_min
ptot_min = p_nco_e_min+9*p_nco_n_min

#Miminum dissipation due to electron and carbon driving
p_e_op = f_re/(y_e*k_x) * Vdd * D_eop * np.ones([b_comp.shape[0]])
p_c_op = f_rc/(y_c*k_x) * Vsup * np.ones([b_comp.shape[0]])
p_static_ac_sig = p_e_op + p_c_op

fig, ax = plt.subplots(2, 1, sharex=True, figsize=(6,4))
ax[0].loglog(b_comp, ptot+p_static_ac_sig, label="Uncompensated field")
# plt.loglog(b_comp, p_dc_magnetic, label="Power dissipation magnetic field")
ax[0].loglog(b_comp, ptot_min+p_dc_magnetic_4k+p_static_ac_sig, label="Compensated field")
ax[0].loglog(b_comp, ptot2+p_dc_magnetic_4k2+p_static_ac_sig, label="Hybrid com.")
# plt.loglog(b_comp, ptot_min, label="Power dissipation Digital QCS compensated field")
# plt.loglog(b_comp, ptot_min+p_dc_magnetic, label="Power dissipation compensated field")
# ax[0].set_title("10 by 10 unit cells", fontproperties=titlefont)
ax[0].set_ylabel("Power dissipation [W]", fontproperties=axfont)
ax[0].set_title("10 by 10 unit cells", fontproperties=titlefont)
ax[0].set_ylim([0.5e-3, 1])
ax[0].set_xlim([1e-1, 1e3])
ax[0].grid()
ax[0].legend(fontsize="medium")