# -*- coding: utf-8 -*-
"""
Created on Wed Aug 31 13:30:53 2022

@author: lucenthoven
"""


import numpy as np
import qutip as qt
import matplotlib.pyplot as plt
import scipy as sc
# from mpl_toolkits.axes_grid.inset_locator import (inset_axes, InsetPosition,
#                                                   mark_inset)
from matplotlib import ticker

from matplotlib import lines
from matplotlib.patches import Rectangle
from matplotlib.transforms import Bbox
import matplotlib.font_manager as fm

plt.style.use("seaborn-v0_8-paper")
# plt.style.use("seaborn-v0_8-paper")
font_path = 'C:\Windows\Fonts\ARIALNB.TTF'  # the location of the font file
# font_path = "liberation-narrow-fonts-ttf-1.07.6\LiberationSansNarrow-Bold.ttf"
suptitlefont = fm.FontProperties(size = 14, fname=font_path)  # get the font based on the font_path
titlefont = fm.FontProperties(size = 12, fname=font_path)  # get the font based on the font_path
axfont = fm.FontProperties(size = 12, fname=font_path)  # get the font based on the font_path

fm.fontManager.addfont(font_path)
prop = fm.FontProperties(fname=font_path)
plt.rcParams['font.family'] = 'sans-serif'
plt.rcParams['font.sans-serif'] = "Arial"
plt.rcParams['font.weight'] = 'bold'
plt.rcParams['font.stretch'] = 'condensed'

plt.rcParams['xtick.labelsize']=12
plt.rcParams['ytick.labelsize']=12

plt.close('all')

# Define constants

y_e= 2.8031e6 # Gyromagnetic ratio Hz/G
y_c = 1.0705e3
y_n = 0.3077e3  # Units

k_coil = 1000 # Coupling factor I->B

D = 2.88e9

Bz = 2e3
inhomogeneity = 0.0012

f_re = 5e6
f_rc = 5e3
theta = np.pi


inf_budget=1e-4/8
print("infidelity allowed per component:", inf_budget)


#%% Excitation frequency (NV-center)

print("Larmor freq e:", np.abs(D-(y_e*Bz)))
print("Larmor freq n:", np.abs((y_c*Bz)))


#%% Operation time
print("e_pi rotation in:", 1/(2*f_re))
print("c_pi rotation in:", 1/(2*f_rc))


#%% Frequency inaccuracy (i.e. finite resolution) 
# Van Dijk 2019 - (Tab 1 carrier freq. inaccuracy)
dMWe = np.sqrt(2*inf_budget/(1-np.cos(np.pi)))*(f_re)
dMWc = np.sqrt(2*inf_budget/(1-np.cos(np.pi)))*(f_rc)
print("allowed frequency inaccuracy dMWe:", dMWe)
print("allowed frequency inaccuracy dMWc:", dMWc)


#%% Frequency noise assuming static error
dMWe = np.sqrt(2*inf_budget/(1-np.cos(np.pi)))*(f_re)
dMWc = np.sqrt(2*inf_budget/(1-np.cos(np.pi)))*(f_rc)
print("allowed frequency noise (static) e:", dMWe)
print("allowed frequency noise (static) c:", dMWc)

#%% Frequency noise - this is lower compared to the inaccuarcy
ENBW = ((2*np.pi*f_re)*(np.pi*np.abs(theta))/(2*(1-np.cos(theta))))/(2*np.pi)
DC_gain = (1-np.cos(theta))/2
# 
MWen = np.sqrt(inf_budget*np.pi*(2*np.pi*f_re)**2)/(2*np.pi)
MWcn = np.sqrt(inf_budget*np.pi*(2*np.pi*f_rc)**2)/(2*np.pi)
print("allowed frequency noise MWe:", MWen)
print("allowed frequency noise MWc:", MWcn)

#%% Frequency wide-band noise 
# Van Dijk 2019 - (Tab 1 wideband noise)
ENBW = ((2*np.pi*f_re)*(2*np.pi*np.abs(theta))/(theta**2 + 2*(1-np.cos(theta))))/(2*np.pi)
DC_gain = (theta**2/4 +(1-np.cos(theta))/2) #squared
# MWwbjeroen = np.sqrt(125e-6/DC_gain)*1e6/0.5e9 # equation matches Jeroens paper
MWwben = np.sqrt(inf_budget/DC_gain)*f_re/y_e # equation mat
MWwbcn = np.sqrt(inf_budget/DC_gain)*f_rc/y_c
print("allowed wide-band frequency dMW:", MWwben)
print("allowed wide-band frequency dMW:", MWwbcn)

# Reference calculation Jeroen DDS-based SoC
# np.sqrt(125e-7/DC_gain)*10e6/0.2e9/(np.sqrt(((2*2*np.pi*f_re)*(2*np.pi*np.abs(theta))/(theta**2 + 2*(1-np.cos(theta))))/(2*np.pi)))

#%% Phase inaccuracy in degrees
# Van Dijk 2019 - (Tab 1 phase inaccuracy)
ph_e = np.sqrt(inf_budget*2/(1-np.cos(theta)))*(180/np.pi)
ph_c = np.sqrt(inf_budget*2/(1-np.cos(theta)))*(180/np.pi)

print("allowed phase error e:", ph_e)
print("allowed phase error n:", ph_c)

#%% Amplitude
# Van Dijk 2019 - (Tab 1 Ampl)
b_ace = np.sqrt(2)*f_re/y_e
b_acn = f_rc/y_c

print("AC field e:", b_ace)
print("AC field n:", b_acn)

dAe = np.sqrt(inf_budget*4/(np.pi**2))*b_ace
dAc = np.sqrt(inf_budget*4/(np.pi**2))*b_acn
print("allowed error dAe:", dAe)
print("allowed error dAc:", dAc)

#%% Time
# Van Dijk 2019 - (Tab 1 duration)
T_e = 1/(2*f_re)
T_c = 1/(2*f_rc)

T_err_e = np.sqrt(inf_budget*4/(np.pi**2))*T_e
T_err_c = np.sqrt(inf_budget*4/(np.pi**2))*T_c
print("allowed error Te:", T_err_e)
print("allowed error Tc:", T_err_c)



#%% Idling infidelity

inf_idle = (1e-4)/4
alpha = 1
beta = np.sqrt(inf_idle*alpha**2 / (np.sin(theta*alpha/2)**2))

print("Beta = ",beta)
print("Beta = ",20*np.log10(beta))

#%% Idling infidelity

inf_idle = (1e-4)/4
omega_spur_e = np.sqrt( (4*inf_idle)/(T_e**2) )
omega_spur_c = np.sqrt( (4*inf_idle)/(T_c**2) )
b_spur_e = np.sqrt(2)* omega_spur_e/(2*np.pi)/y_e
b_spur_c = omega_spur_c/(2*np.pi)/y_c

print("spur e:", b_spur_e)
print("spur c:", b_spur_c)


#%% Idling Z-field inaccuracy

inf_idle_e = np.arccos(np.sqrt(1-inf_idle))*2/T_e
inf_idle_c = np.arccos(np.sqrt(1-inf_idle))*2/T_c


# print("Z-field inaccuracy e(Hz):", inf_idle_e/(2*np.pi))
print("Z-field inaccuracy e:", inf_idle_e/(2*np.pi)/y_e)
print("Z-field inaccuarcy C:", inf_idle_c/(2*np.pi)/y_c)



#%% DC Z Noise infidelity in Gauss^2/Hz
ENBW = 1/(2*T_e)
DC_gain = (T_e**2)/2
ENBW_c = 1/(2*T_c)
DC_gain_c = (T_c**2)/2
MWen_dc = inf_idle*np.pi/(ENBW*DC_gain*(2*np.pi*y_e)**2)
MWcn_dc = inf_idle*np.pi/(ENBW_c*DC_gain_c*(2*np.pi*y_c)**2)
print("allowed noise DCZe:", MWen_dc)
print("allowed noise DCZc:", MWcn_dc)

#%% DC x/y Noise infidelity in Gauss^2/Hz
ENBW = 1/T_e
DC_gain = (T_e**2)/2
ENBW_c = 1/T_c
DC_gain_c = (T_c**2)/2
MWen_dc = inf_idle*np.pi/(ENBW*DC_gain*(2*np.pi*y_e)**2)
MWcn_dc = inf_idle*np.pi/(ENBW_c*DC_gain_c*(2*np.pi*y_c)**2)
print("allowed noise DCxye:", MWen_dc)
print("allowed noise DCxyc:", MWcn_dc)

#%% Amplitude in current domain
kx = 290
i_ace = b_ace/kx
i_acn = b_acn/kx

print("AC current e:", i_ace)
print("AC current n:", i_acn)

dIe = dAe/kx
dIc = dAc/kx
print("allowed error dIe:", dIe)
print("allowed error dIc:", dIc)

print("allowed wide-band frequency dMW in current:", MWwben/kx)
print("allowed wide-band frequency dMW in current:", MWwbcn/kx)
#%% Amplitude in current domain
kz = 710

print("DC Z-field:", Bz*inhomogeneity/kz)

print("DC dcurrent e:", inf_idle_e/(2*np.pi)/y_e/kz)
print("DC dcurrent n:", inf_idle_c/(2*np.pi)/y_c/kz)

ENBW = 1/(2*T_e)
DC_gain = (T_e**2)/2
ENBW_c = 1/(2*T_c)
DC_gain_c = (T_c**2)/2
MWen_dc = inf_idle*np.pi/(ENBW*DC_gain*(2*np.pi*y_e)**2)
MWcn_dc = inf_idle*np.pi/(ENBW_c*DC_gain_c*(2*np.pi*y_c)**2)
print("allowed noise DCZe:", MWen_dc/(kz**2))
print("allowed noise DCZc:", MWcn_dc/(kz**2))

ENBW = 1/T_e
DC_gain = (T_e**2)/2
ENBW_c = 1/T_c
DC_gain_c = (T_c**2)/2
MWen_dc = inf_idle*np.pi/(ENBW*DC_gain*(2*np.pi*y_e)**2)
MWcn_dc = inf_idle*np.pi/(ENBW_c*DC_gain_c*(2*np.pi*y_c)**2)
print("allowed noise DCxye:", MWen_dc/(kz**2))
print("allowed noise DCxyn:", MWcn_dc/(kz**2))



#%% Overlap excited state and ground state for different magnetic fields
# Used to compute the orthogonal magnetic field allowed

#Coefficients
D = 2.88
Des_par = 1.42
Des_perp = 1.55/2
lambda_es_par = 5.3
lambda_es_perp = 0.14
l_es_par = 0.1
g_e = 2
hbar = 1.0545718e-34  # Units
h = hbar * 2*np.pi
u_b = 9.274e-24
u_b_red = g_e*u_b/hbar/1e9/10000/(2*np.pi) #per gauss in GHz
y_e_2 = g_e*u_b/hbar/(2*np.pi)
y_e = 2.802e6 #per G
y_e_g = 2.802e-3 #per G

sx = qt.jmat(1, "x")
sy = qt.jmat(1, "y")
sz = qt.jmat(1, "z")
ox = qt.jmat(0.5, "x")*2
oy = qt.jmat(0.5, "y")*2
oz = qt.jmat(0.5, "z")*2
s = qt.identity(3)
I = qt.identity(2)


rhop1 = (qt.basis(3,0)*qt.basis(3,0).dag())
rho0 = (qt.basis(3,1)*qt.basis(3,1).dag())
rhom1 = (qt.basis(3,2)*qt.basis(3,2).dag())
error_angle = np.logspace(-3, 1.5, 1000) #degrees converted to radians
rad_angle = error_angle*np.pi/180 #degrees converted to radians

bmag = 10000 # 1 Tesla Field
min_overlap_arr_1T = []
bx_1T = []

for ind, val in enumerate(rad_angle): 
    bz = bmag * np.cos(val)
    br = bmag * np.sin(val)
    
    H = D*sz*sz + bz*y_e_g*sz + br*y_e_g*sx
    H_es = Des_par*(qt.tensor(I, sz*sz)) - lambda_es_par*qt.tensor(oy, sz) \
        + Des_perp*(qt.tensor(oz, (sy*sy-sx*sx))-qt.tensor(ox, (sy*sx + sx*sy))) \
        + lambda_es_perp*(qt.tensor(oz, (sx*sz+sz*sx))-qt.tensor(ox, (sy*sz + sz*sy))) \
        + bz*y_e_g * ((1/g_e)*l_es_par * qt.tensor(oy,s) + qt.tensor(I,sz)) + br*y_e_g*qt.tensor(I,sx)
        
    es_eig_s = (H_es.ptrace(1)).eigenstates()
    eig_s = H.eigenstates()
    min_eig = 1.0
    for i in range(3):
        overlap = abs(np.real(eig_s[1][i].overlap(es_eig_s[1][i])))
        if overlap < min_eig:
            min_eig = overlap
    min_overlap_arr_1T.append(min_eig)
    bx_1T.append(br)


min_overlap_arr_2000 = []
bx_2000 = []
bmag = 2000

for ind, val in enumerate(rad_angle): 
    bz = bmag * np.cos(val)
    br = bmag * np.sin(val)
    
    H = D*sz*sz + bz*y_e_g*sz + br*y_e_g*sx
    H_es = Des_par*(qt.tensor(I, sz*sz)) - lambda_es_par*qt.tensor(oy, sz) \
        + Des_perp*(qt.tensor(oz, (sy*sy-sx*sx))-qt.tensor(ox, (sy*sx + sx*sy))) \
        + lambda_es_perp*(qt.tensor(oz, (sx*sz+sz*sx))-qt.tensor(ox, (sy*sz + sz*sy))) \
        + bz*y_e_g * ((1/g_e)*l_es_par * qt.tensor(oy,s) + qt.tensor(I,sz)) + br*y_e_g*qt.tensor(I,sx)
         
    es_eig_s = (H_es.ptrace(1)).eigenstates()
    eig_s = H.eigenstates()
    min_eig = 1.0
    for i in range(3):
        overlap = abs(np.real(eig_s[1][i].overlap(es_eig_s[1][i])))
        if overlap < min_eig:
            min_eig = overlap
    min_overlap_arr_2000.append(min_eig)
    bx_2000.append(br)
    
min_overlap_arr_5000 = []
bx_5000 = []
bmag = 5000

for ind, val in enumerate(rad_angle): 
    bz = bmag * np.cos(val)
    br = bmag * np.sin(val)
    
    H = D*sz*sz + bz*y_e_g*sz + br*y_e_g*sx
    H_es = Des_par*(qt.tensor(I, sz*sz)) - lambda_es_par*qt.tensor(oy, sz) \
        + Des_perp*(qt.tensor(oz, (sy*sy-sx*sx))-qt.tensor(ox, (sy*sx + sx*sy))) \
        + lambda_es_perp*(qt.tensor(oz, (sx*sz+sz*sx))-qt.tensor(ox, (sy*sz + sz*sy))) \
        + bz*y_e_g * ((1/g_e)*l_es_par * qt.tensor(oy,s) + qt.tensor(I,sz)) + br*y_e_g*qt.tensor(I,sx)
         
    es_eig_s = (H_es.ptrace(1)).eigenstates()
    eig_s = H.eigenstates()
    min_eig = 1.0
    for i in range(3):
        overlap = abs(np.real(eig_s[1][i].overlap(es_eig_s[1][i])))
        if overlap < min_eig:
            min_eig = overlap
    min_overlap_arr_5000.append(min_eig)
    bx_5000.append(br)
    


bmag = 400

min_overlap_arr_400 = []
bx_400 = []
for ind, val in enumerate(rad_angle): 
    bz = bmag * np.cos(val)
    br = bmag * np.sin(val)
    
    H = D*sz*sz + bz*y_e_g*sz + br*y_e_g*sx
    H_es = Des_par*(qt.tensor(I, sz*sz)) - lambda_es_par*qt.tensor(oy, sz) \
        + Des_perp*(qt.tensor(oz, (sy*sy-sx*sx))-qt.tensor(ox, (sy*sx + sx*sy))) \
        + lambda_es_perp*(qt.tensor(oz, (sx*sz+sz*sx))-qt.tensor(ox, (sy*sz + sz*sy))) \
        + bz*y_e_g * ((1/g_e)*l_es_par * qt.tensor(oy,s) + qt.tensor(I,sz)) + br*y_e_g*qt.tensor(I,sx)
       
    es_eig_s = (H_es.ptrace(1)).eigenstates()
    eig_s = H.eigenstates()
    min_eig = 1.0
    for i in range(3):
        # if i == 0:
        #     overlap = abs(np.real(eig_s[1][1].overlap(es_eig_s[1][0])))
        # if i == 1:
        #     overlap = abs(np.real(eig_s[1][0].overlap(es_eig_s[1][1])))
        # if i == 2:
        #     overlap = abs(np.real(eig_s[1][i].overlap(es_eig_s[1][i])))
        overlap = abs(np.real(eig_s[1][i].overlap(es_eig_s[1][i])))
        if overlap < min_eig:
            min_eig = overlap
    min_overlap_arr_400.append(min_eig)
    bx_400.append(br)
    

bmag = 45

min_overlap_arr_45 = []
bx_45 = []
for ind, val in enumerate(rad_angle): 
    bz = bmag * np.cos(val)
    br = bmag * np.sin(val)
    
    H = D*sz*sz + bz*y_e_g*sz + br*y_e_g*sx
    H_es = Des_par*(qt.tensor(I, sz*sz)) - lambda_es_par*qt.tensor(oy, sz) \
        + Des_perp*(qt.tensor(oz, (sy*sy-sx*sx))-qt.tensor(ox, (sy*sx + sx*sy))) \
        + lambda_es_perp*(qt.tensor(oz, (sx*sz+sz*sx))-qt.tensor(ox, (sy*sz + sz*sy))) \
        + bz*y_e_g * ((1/g_e)*l_es_par * qt.tensor(oy,s) + qt.tensor(I,sz)) + br*y_e_g*qt.tensor(I,sx)
      
    es_eig_s = (H_es.ptrace(1)).eigenstates()
    eig_s = H.eigenstates()
    min_eig = 1.0
    for i in range(3):
        overlap = abs(np.real(eig_s[1][i].overlap(es_eig_s[1][i])))
        if overlap < min_eig:
            min_eig = overlap
    min_overlap_arr_45.append(min_eig)
    bx_45.append(br)
    

plt.figure()
plt.plot(error_angle, min_overlap_arr_1T, label=r'10000 Gauss')
plt.plot(error_angle, min_overlap_arr_2000, label=r'2000 Gauss')
plt.plot(error_angle, min_overlap_arr_5000, label=r'5000 Gauss')
plt.plot(error_angle, min_overlap_arr_400, label=r'400 Gauss')
plt.plot(error_angle, min_overlap_arr_45, label=r'45 Gauss')
plt.xlabel("Error angle")
plt.ylabel("Minimum overlap spin states")
plt.legend(loc="lower right")
plt.grid(axis="both")
plt.ylim([0.99, 1])
plt.title("Error angle versus normalized eigentate component for different $B_z$")

# ax1 = plt.gca()
# ax3 = inset_axes(ax1, width="20%", height="20%", loc="upper left")
# ip2 = InsetPosition(ax1, [0.15,0.2,0.4,0.4])
# ax3.set_axes_locator(ip2)
# ax3.plot(error_angle, min_overlap_arr_1T)
# ax3.plot(error_angle, min_overlap_arr_5000)
# ax3.plot(error_angle, min_overlap_arr_2000)
# ax3.plot(error_angle, min_overlap_arr_400)
# ax3.plot(error_angle, min_overlap_arr_45)
# ax3.set_xlim([0, 0.5])
# ax3.set_ylim([0.999995, 1])
# ax3.yaxis.set_major_formatter(ticker.StrMethodFormatter("{x:.6f}"))
# ax3.grid(axis="both")
# plt.setp(ax3.spines.values(), color='black')
# mark_inset(ax1, ax3, loc1=1, loc2=3, edgecolor='black', linewidth=1)

plt.tight_layout()
# plt.savefig("figures/eigenstate_simulation_es_gs.png", dpi=300)


plt.figure()
plt.plot(bx_1T, min_overlap_arr_1T, label=r'10000 Gauss magnet')
plt.plot(bx_5000, min_overlap_arr_5000, label=r'5000 Gauss')
plt.plot(bx_2000, min_overlap_arr_2000, label=r'2000 Gauss')
plt.plot(bx_400, min_overlap_arr_400, label=r'400 Gauss')
plt.plot(bx_45, min_overlap_arr_45, label=r'45 Gauss magnet')
plt.xlabel("$B_R$ [G]")
plt.ylabel("Minimum overlap spin states")
plt.title("$B_R$ versus normalized eigentate component for different $B_z$")
plt.legend()
plt.xlim([0, 50])
plt.ylim([0.999999, 1.00000025])
plt.tight_layout()
plt.grid(axis="both")
plt.savefig("figures/eigenstate_simulation_es_gs_br_comp.png", dpi=300)

readout_cycles = 100
r_inf_1T = 1-(np.array(min_overlap_arr_1T)**readout_cycles)
r_inf_5000 = 1-(np.array(min_overlap_arr_5000)**readout_cycles)
r_inf_2000 = 1-(np.array(min_overlap_arr_2000)**readout_cycles)
r_inf_400 = 1-(np.array(min_overlap_arr_400)**readout_cycles)
r_inf_45 = 1-(np.array(min_overlap_arr_45)**readout_cycles)

plt.figure()
plt.loglog(error_angle, r_inf_1T, label=r'10000 Gauss')
plt.loglog(error_angle, r_inf_5000, label=r'5000 Gauss')
plt.loglog(error_angle, r_inf_2000, label=r'2000 Gauss')
plt.loglog(error_angle, r_inf_400, label=r'400 Gauss')
plt.loglog(error_angle, r_inf_45, label=r'45 Gauss')
plt.hlines(1e-4, 0.1, 10)
plt.xlabel("Error angle")
plt.ylabel("Readout infidelity")
plt.legend()
plt.grid(axis="both")
plt.title("Error angle versus readout infidelity for {} cycles".format(readout_cycles))
plt.savefig("figures/eigenstate_simulation_conv_readout_infidelity.png", dpi=300)

plt.figure(figsize=(5,4))
plt.loglog(bx_1T, r_inf_1T, label=r'10000 Gauss')
plt.loglog(bx_5000, r_inf_5000, label=r'5000 Gauss')
plt.loglog(bx_2000, r_inf_2000, label=r'2000 Gauss')
plt.loglog(bx_400, r_inf_400, label=r'400 Gauss')
plt.loglog(bx_45, r_inf_45, label=r'45 Gauss')
plt.hlines(1e-4, 0.01, 1000, color="black")
plt.xlabel("$\mathregular{B_{perp}}$ [G]", fontproperties=axfont)
plt.ylabel("Readout infidelity", fontproperties=axfont)
plt.xlim([0.1, 1000])
plt.ylim([1e-5, 1e-2])
plt.legend()
plt.grid(axis="both")
plt.title("$\mathregular{B_{perp}}$ versus readout infidelity for " + "{} cycles".format(readout_cycles), fontproperties=axfont)
plt.tight_layout()
plt.savefig("figures/Fig_4_eigenstate_simulation_conv_readout_infidelity_mag_field.png", dpi=300)
plt.savefig("figures/Fig_4_eigenstate_simulation_conv_readout_infidelity_mag_field.eps", format="eps")

# %% Infidelity due to soft driving
# Sweep crosstalk vs frequency spacing

T_op_sweep = np.logspace(-8, -5, 1000)

w_r_addr = f_re*2*np.pi
# I_coil = w_r_addr/(y_e*k_x_addr*2*np.pi)
I_coil = 1/50 # assume 1Vpk signal with 50 ohm termination
k_x_unaddr_1m = 0.1 #simulated Kx for 500um distance
w_r_unaddr_1m = 2*np.pi*y_e*k_x_unaddr_1m*I_coil

print("Beta:", w_r_unaddr_1m/w_r_addr)

f_space = np.logspace(4, 8, 5)


plt.figure(figsize=(5,4))
F_coupling_crosstalk_diff_1m = np.cos(w_r_unaddr_1m*T_op_sweep/2)**2
# plt.loglog(T_op_sweep, 1-F_coupling_crosstalk_diff_1m, label="$\mathregular{f_{space}}$ = 0")

for ind, val in enumerate(f_space):
    w_space_pi = val * 2*np.pi
    inf_alpha = (w_r_unaddr_1m**2)/(w_space_pi**2)*np.sin(T_op_sweep*w_space_pi/2)**2
    plt.loglog(T_op_sweep, inf_alpha+1e-14, label="$\mathregular{f_{space}}$ = "+"{:.0e} Hz".format(val))


plt.hlines(1e-5, 1e-8, 1e-5, color="black")
# plt.title("Infidelity due to soft driving")
plt.xlabel("t [S]", fontproperties=axfont)
plt.ylabel("Infidelty", fontproperties=axfont)
plt.legend()
plt.grid(axis="both")
plt.ylim([1e-9, 1e-1])
plt.xlim([1e-8, 1e-5])
plt.tight_layout()
plt.savefig("figures/Fig_7_crosstalk_infidelity_w0space_soft_driving.png", dpi=300)
plt.savefig("figures/Fig_7_crosstalk_infidelity_w0space_soft_driving.eps", format="eps")