# -*- coding: utf-8 -*-
"""
Created on Thu Nov 23 13:52:34 2023

@author: lucenthoven
"""

import numpy as np
import qutip as qt
import matplotlib.pyplot as plt
import scipy as sc
from scipy import signal
# from mpl_toolkits.axes_grid.inset_locator import (inset_axes, InsetPosition,
#                                                   mark_inset)
from matplotlib import ticker

from matplotlib import lines
from matplotlib.patches import Rectangle
from matplotlib.transforms import Bbox
import matplotlib.font_manager as fm

plt.close("all")

def hermite_env(t_arr):
    """
    Gives the hermite envelope that is typically used
    Returns
    -------
    herm_env : Numpy array
        Hermite envelope.

    """
    tmax = np.max(t_arr)
    duration = tmax
    T_herm = 0.1667 * duration
    x = ((t_arr - duration / 2) / T_herm) ** 2
    herm_env = (1 - 0.956 * x) * np.exp(-x)
    return herm_env


def gauss_envelope(t_arr, sigm=6):
    """
    Gives an gaussian envelope
    Returns
    -------
    Numpy array
        Gaussian envelope.

    """
    N = t_arr.shape[0]
    env = signal.windows.gaussian(N, std=(N / sigm))
    return env

#Single electron pulse with Hermite shape
t = np.linspace(0, 100e-9, num=1000)
f0 = 100e6
sig = np.cos(2*np.pi*f0*t)
env = hermite_env(t)
shaped_pulse = sig*env

plt.figure(1)
# plt.plot(t, sig, color="C0", linewidth=2)
plt.plot(t, sig*env, color="C0", linewidth=0.5)
plt.plot(t, env, color="black", linewidth=2)
plt.savefig("figures/electron_pulse.eps", format="eps")

t_e = t
sig_e = sig
env_e = env
shaped_pulse_e = shaped_pulse
#Single electron pulse with Hermite shape with 'tau' delay


t2 = np.linspace(0, 100e-9, num=1000)
zeros = np.zeros(t2.shape)
t_shift = np.max(t2)+t

dd_t_ax = np.r_[t2, t_shift, np.max(t_shift)+t2]
dd_y_ax = np.r_[zeros, shaped_pulse, zeros]
dd_y_env = np.r_[zeros, env, zeros]

plt.figure(2)
# plt.plot(t, sig, color="C0", linewidth=2)
plt.plot(dd_t_ax, dd_y_ax, color="C0", linewidth=0.5)
plt.plot(dd_t_ax, dd_y_env, color="black", linewidth=2)
plt.text(50e-9, 0.5, "$\\tau$", fontsize=20)
plt.text(250e-9, 0.5, "$\\tau$", fontsize=20)
plt.savefig("figures/electron__tau_pulse.eps", format="eps")


#Single electron pulse with Hermite shape
t = np.linspace(0, 300e-9, num=1000)
f0 = 10e6
sig = np.cos(2*np.pi*f0*t)
pad_zeros = 50
ones_res = t.shape[0]-2*pad_zeros
env = np.r_[np.zeros(pad_zeros), np.ones([ones_res]), np.zeros(pad_zeros)]
shaped_pulse = sig*env

plt.figure(3)
# plt.plot(t, sig, color="C0", linewidth=2)
plt.plot(t, sig*env, color="C1", linewidth=0.5)
plt.plot(t, env, color="black", linewidth=2)
plt.savefig("figures/nuclear_pulse.eps", format="eps")

#Single electron pulse with Hermite shape
t = np.linspace(0, 100e-9, num=1000)
f0 = 20e6
sig = np.cos(2*np.pi*f0*t)
pad_zeros = 0
ones_res = t.shape[0]-2*pad_zeros
env = np.r_[np.zeros(pad_zeros), np.ones([ones_res]), np.zeros(pad_zeros)]
shaped_pulse = sig*env
t_e_shift = t_e + np.max(t)
zeros_n = np.zeros([shaped_pulse.shape[0]])

dd_t_ax = np.r_[t, t_e_shift, np.max(t_e_shift)+t]
dd_y_ax_e = np.r_[zeros_n, shaped_pulse_e, zeros_n]
dd_y_ax_N = np.r_[shaped_pulse, np.zeros([shaped_pulse_e.shape[0]]), shaped_pulse]
dd_y_env_e = np.r_[zeros_n, env_e, zeros_n]
dd_y_env_N = np.r_[env, np.zeros(env_e.shape[0]), env]

plt.figure(4)
# plt.plot(t, sig, color="C0", linewidth=2)
plt.plot(dd_t_ax, dd_y_ax_e, color="C0", linewidth=0.5)
plt.plot(dd_t_ax, dd_y_ax_N, color="C1", linewidth=0.5)
plt.plot(dd_t_ax, dd_y_env_e, color="gray", linewidth=2)
plt.plot(dd_t_ax, dd_y_env_N, color="black", linewidth=2)
plt.text(50e-9, 1.1, "$\\tau$", fontsize=20)
plt.text(250e-9, 1.1, "$\\tau$", fontsize=20)
plt.savefig("figures/electron_ddrf.eps", format="eps")


t = np.linspace(0, 100e-9, num=1000)
f0 = 100e6
sig = np.cos(2*np.pi*f0*t)
env = gauss_envelope(t)
shaped_pulse = sig*env

plt.figure(6)
# plt.plot(t, sig, color="C0", linewidth=2)
plt.plot(t, sig*env, color="C0", linewidth=0.5)
plt.plot(t, env, color="black", linewidth=2)
plt.savefig("figures/electron_pulse_gauss.eps", format="eps")
